@@ -1,9 +1,9 @@
 #' @title
-#' Create \code{trans_venn} object.
+#' Create \code{trans_venn} object for the Venn diagram and UpSet plot.
 #'
 #' @description
-#' This class is a wrapper for a series of venn analysis related methods, including 2- to 5-way venn diagram, 
-#' more than 5-way petal or bar plot and intersection transformations based on David et al. (2012) <doi:10.1128/AEM.01459-12>.
+#' This class is a wrapper for a series of intersection analysis related methods, including 2- to 5-way venn diagram, 
+#' more than 5-way petal or UpSet plot and intersection transformations based on David et al. (2012) <doi:10.1128/AEM.01459-12>.
 #'
 #' @export
 trans_venn <- R6Class(classname = "trans_venn",
@@ -48,14 +48,15 @@ trans_venn <- R6Class(classname = "trans_venn",
 			res_names <- colnames(abund)
 			colnumber <- ncol(abund)
 			abund1 <- cbind.data.frame(OTU = rownames(abund), abund)
-			abund2 <- reshape2::melt(abund1, id.var = "OTU", value.name= "Abundance", variable.name = "SeqID")
-			## Create intersect matrix (removes duplicates)
+			abund2 <- reshape2::melt(abund1, id.var = "OTU", value.name = "Abundance", variable.name = "SeqID")
+			# Create intersection matrix (removes duplicates)
 			setunion <- rownames(abund)
 			setmatrix <- abund
 			setmatrix[setmatrix >= 1] <- 1
-			## Create all possible sample combinations within requested complexity levels
-			# modified from the code of systemPipeR package 
-			allcombl <- lapply(1:colnumber, function(x) combn(colnames(setmatrix), m = x, simplify = FALSE)) %>% unlist(recursive=FALSE)
+			samplesum <- apply(setmatrix, 2, sum)
+			# Create all possible sample combinations within requested complexity levels
+			# modified from the method of systemPipeR package
+			allcombl <- lapply(1:colnumber, function(x) combn(colnames(setmatrix), m = x, simplify = FALSE)) %>% unlist(recursive = FALSE)
 			venn_list <- sapply(seq_along(allcombl), function(x) private$vennSets(setmatrix = setmatrix, allcombl = allcombl, index = x, setunion = setunion))
 			names(venn_list) <- sapply(allcombl, paste, collapse = name_joint)
 			venn_abund <- sapply(venn_list, function(x){
@@ -84,6 +85,7 @@ trans_venn <- R6Class(classname = "trans_venn",
 			venn_table <- as.data.frame(t(do.call(rbind, venn_table)))
 			self$data_summary <- venn_count_abund
 			self$data_details <- venn_table
+			self$data_samplesum <- samplesum
 			self$colnumber <- colnumber
 			self$res_names <- res_names
 			self$ratio <- ratio
@@ -333,32 +335,45 @@ trans_venn <- R6Class(classname = "trans_venn",
 			p
 		},
 		#' @description
-		#' Plot the intersections using histogram. Especially useful when samples > 5.
+		#' Plot the intersections using , i.e. UpSet plot. Especially useful when samples > 5.
 		#'
-		#' @param bottom_y_text_size default 12; y axis text size, i.e. sample name size, of bottom plot.
+		#' @param left_plot default TRUE; whether add the left bar plot to show the feature number of each sample.
+		#' @param sort_samples default TRUE; whether sort samples according to the number of features in each sample.
+		#'   If FALSE, use the sample orders in sample_table of the raw dataset.
 		#' @param up_y_title default "Intersection set"; y axis title of upper plot.
 		#' @param up_y_title_size default 15; y axis title size of upper plot.
 		#' @param up_y_text_size default 4; y axis text size of upper plot.
-		#' @param bar_color default "grey70"; bar border color of upper plot.
-		#' @param bar_fill default "grey70"; bar fill color of upper plot.
-		#' @param point_size default 3; point size of bottom plot.
-		#' @param point_color default "black"; point color of bottom plot.
-		#' @param bottom_height default 0.5; bottom plot height.
+		#' @param up_bar_fill default "grey70"; bar fill color of upper plot.
+		#' @param bottom_y_text_size default 12; y axis text size, i.e. sample name size, of bottom sample plot.
+		#' @param bottom_height default 1; bottom plot height relative to the upper bar plot. 1 represents the height of bottom plot is same with the upper bar plot.
+		#' @param bottom_point_size default 3; point size of bottom plot.
+		#' @param bottom_point_color default "black"; point color of bottom plot.
+		#' @param bottom_background_fill default "grey95"; fill color for the striped background in the bottom sample plot.
+		#' @param left_width default 0.3; left bar plot width relative to the right bottom plot.
+		#' @param left_bar_fill default "grey70"; fill color for the left bar plot presenting feature number.
+		#' @param left_x_text_size default 10; x axis text size of the left bar plot.
+		#' @param left_background_fill default "grey95"; fill color for the striped background in the left plot.
 		#' @return a ggplot2 object.
 		#' @examples
 		#' \donttest{
 		#' t2 <- t1$plot_bar()
 		#' }
 		plot_bar = function(
-			bottom_y_text_size = 12,
-			up_y_title = "Intersection set",
+			left_plot = TRUE,
+			sort_samples = TRUE,
+			up_y_title = "Intersection size",
 			up_y_title_size = 15,
 			up_y_text_size = 8,
-			bar_color = "grey70",
-			bar_fill = "grey70",
-			point_size = 3,
-			point_color = "black",
-			bottom_height = 0.5
+			up_bar_fill = "grey70",
+			bottom_y_text_size = 12,
+			bottom_height = 1,
+			bottom_point_size = 3,
+			bottom_point_color = "black",
+			bottom_background_fill = "grey95",
+			left_width = 0.3,
+			left_bar_fill = "grey70",
+			left_x_text_size = 10,
+			left_background_fill = "grey95"
 			){
 			colnumber <- self$colnumber
 			ratio <- self$ratio
@@ -366,10 +381,16 @@ trans_venn <- R6Class(classname = "trans_venn",
 			switch_num <- colnumber-1
 			summary_table <- self$data_summary
 			name_joint <- self$name_joint
+			samplesum <- self$data_samplesum
 			
 			if(any(grepl(name_joint, res_names, fixed = TRUE))){
 				stop("Please change name_joint parameter when creating trans_venn object!")
 			}
+			if(sort_samples){
+				sample_levels <- sort(samplesum, decreasing = TRUE) %>% names %>% rev
+			}else{
+				sample_levels <- res_names %>% rev
+			}
 			
 			plot_data <- summary_table %>% 
 				rownames_to_column %>% 
@@ -378,38 +399,85 @@ trans_venn <- R6Class(classname = "trans_venn",
 			
 			g1 <- ggplot(plot_data, aes(x = rowname, y = Counts)) +
 				theme_classic() +
-				geom_col(color = bar_color, fill = bar_fill) +
+				geom_col(color = up_bar_fill, fill = up_bar_fill) +
 				ylab(up_y_title) +
 				theme(axis.title.x = element_blank(), axis.text.x = element_blank(), axis.ticks.x = element_blank()) +
 				theme(axis.text = element_text(size = up_y_text_size), axis.title = element_text(size = up_y_title_size))
 			
-			data2 <- matrix(nrow = colnumber, ncol = nrow(plot_data)) %>% as.data.frame
-			rownames(data2) <- res_names
-			colnames(data2) <- plot_data[, 1]
-			for(i in colnames(data2)){
+			matrix_data <- matrix(nrow = colnumber, ncol = nrow(plot_data)) %>% as.data.frame
+			rownames(matrix_data) <- sample_levels
+			colnames(matrix_data) <- plot_data[, 1]
+			for(i in colnames(matrix_data)){
 				tmp <- strsplit(i, name_joint, fixed = TRUE) %>% unlist
-				for(j in rownames(data2)){
+				for(j in rownames(matrix_data)){
 					if(j %in% tmp){
-						data2[j, i] <- 1
+						matrix_data[j, i] <- 1
 					}
 				}
 			}
-			data2 %<>% rownames_to_column %>% reshape2::melt(., id.vars = "rowname")
-			data2$variable %<>% factor(., levels = levels(plot_data[, 1]))
-			data3 <- data2[!is.na(data2$value), ]
-			#data2$value[is.na(data2$value)] <- 0
-			g2 <- ggplot(data2, aes(x = variable, y = rowname)) +
+			sample_long <- matrix_data %>% rownames_to_column %>% reshape2::melt(., id.vars = "rowname")
+			sample_long$variable %<>% factor(., levels = levels(plot_data[, 1]))
+			sample_long$rowname %<>% factor(levels = sample_levels)
+			sample_ture <- sample_long[!is.na(sample_long$value), ]
+			sample_all <- sample_long
+			sample_all$value <- 1
+
+			g2 <- ggplot(sample_long, aes(x = variable, y = rowname))
+				#theme(plot.background = element_rect(fill = "white", colour = "white", size = 0.1))
+			for(i in seq_len(length(unique(sample_ture$rowname)))){
+				if(i %% 2 == 1){
+					g2 <- g2 + geom_rect(ymin = i - 0.5, ymax = i + 0.5, xmin = -Inf, xmax = Inf, fill = bottom_background_fill, colour = bottom_background_fill)
+				}else{
+					g2 <- g2 + geom_rect(ymin = i - 0.5, ymax = i + 0.5, xmin = -Inf, xmax = Inf, fill = "white", colour = "white")
+				}
+			}
+			g2 <- g2 + 
+				geom_point(aes(x = variable, y = rowname), data = sample_all, size = bottom_point_size, color = "grey92", inherit.aes = FALSE) +
+				geom_point(aes(x = variable, y = rowname), data = sample_ture, size = bottom_point_size, color = bottom_point_color, inherit.aes = FALSE) +
 				theme_bw() +
-				geom_point(aes(x = variable, y = rowname), data = data3, size = point_size, color = point_color, inherit.aes = FALSE) +
 				theme(legend.position = "none") +
 				theme(axis.title = element_blank(), axis.text.x = element_blank(), axis.ticks = element_blank()) +
 				theme(axis.text = element_text(size = bottom_y_text_size)) +
 				theme(panel.border = element_blank()) +
-				theme(panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank())
-				#theme(plot.background = element_rect(fill = "white", colour = "white", size = 0.1))
+				theme(panel.grid = element_blank())
+			
+			line_data <- matrix_data
+			line_data[] <- lapply(line_data, function(x){x <- 1:length(x); x})
+			line_data[is.na(matrix_data)] <- NA
 
-			g3 <- aplot::insert_bottom(g1, g2, height = bottom_height)
-			g3
+			line_data2 <- data.frame(y = apply(line_data, 2, min, na.rm = TRUE), yend = apply(line_data, 2, max, na.rm = TRUE), x = 1:ncol(line_data), xend = 1:ncol(line_data))
+			g2 <- g2 + geom_segment(aes(x = x, y = y, xend = xend, yend = yend), data = line_data2)
+
+			if(left_plot){
+				g3_data <- data.frame(number = samplesum, rowname = names(samplesum))
+				g3_data$rowname %<>% factor(levels = sample_levels)
+
+				g3 <- ggplot(g3_data, aes(x = number, y = rowname))
+				for(i in seq_len(length(unique(g3_data$rowname)))){
+					if(i %% 2 == 1){
+						g3 <- g3 + geom_rect(ymin = i - 0.5, ymax = i + 0.5, xmin = -Inf, xmax = Inf, fill = left_background_fill, colour = left_background_fill)
+					}else{
+						g3 <- g3 + geom_rect(ymin = i - 0.5, ymax = i + 0.5, xmin = -Inf, xmax = Inf, fill = "white", colour = "white")
+					}
+				}
+				g3 <- g3 + 
+					geom_col(color = left_bar_fill, fill = left_bar_fill) +
+					theme_bw() +
+					theme(legend.position = "none") +
+					scale_x_reverse() +
+					theme(axis.title = element_blank(), axis.text.y = element_blank(), axis.ticks.y = element_blank()) +
+					theme(axis.text = element_text(size = left_x_text_size)) +
+					theme(panel.border = element_blank()) +
+					theme(panel.grid = element_blank())
+				
+				p1 <- g2 %>% 
+					aplot::insert_top(g1, height = 1/bottom_height) %>% 
+					aplot::insert_left(g3, width = left_width)
+				p1
+			}else{
+				p1 <- aplot::insert_bottom(g1, g2, height = bottom_height)
+				p1
+			}
 		},
 		#' @description
 		#' Transform venn result to community-like microtable object for further composition analysis.
@@ -498,13 +566,13 @@ trans_venn <- R6Class(classname = "trans_venn",
 			ang <- ang[1:n]
 			ang <- ang*pi/180
 			x <- r * cos(ang)
-			y <- r* sin(ang)
-			xy <- rbind(x,y)
-			m <- diag(c(a,b))
+			y <- r * sin(ang)
+			xy <- rbind(x, y)
+			m <- diag(c(a, b))
 			xy <- m %*% xy
 			rotate <- rotate*pi/180
 			m <- c(cos(rotate), sin(rotate), -sin(rotate), cos(rotate))
-			dim(m) <- c(2,2)
+			dim(m) <- c(2, 2)
 			xy <- m %*% xy
 			xy[1, ] <- xy[1, ] + mx
 			xy[2, ] <- xy[2, ] + my
